from fastapi import APIRouter,Depends,Request,HTTPException
from sqlalchemy.orm import Session
from typing import Dict, Any
from brave.api.service.pipeline  import get_all_module
# from brave.api.config.db import conn
# from models.user import users
from typing import List
from starlette.status import HTTP_204_NO_CONTENT
from sqlalchemy import func, select
from brave.api.models.orm import SampleAnalysisResult
import glob
import importlib
import os
from brave.api.config.db import get_db_session
from sqlalchemy import and_,or_
from io import BytesIO
import base64
import json
import traceback
import uuid
import pandas as pd
from brave.api.config.db import get_engine
from brave.api.models.core import samples
import inspect
# from brave.api.routes.analysis import get_db_value
from brave.api.utils.get_db_utils import get_ids
file_parse_plot = APIRouter()
from brave.api.config.config import get_settings
from pathlib import Path
from brave.api.routes.sample_result import find_analyais_result_by_ids

# key = Fernet.generate_key()
# f = Fernet(key)


def get_all_subclasses(cls):
    subclasses = set(cls.__subclasses__())
    for subclass in subclasses.copy():
        subclasses.update(get_all_subclasses(subclass))
    return subclasses



def get_sample(project):
    with get_engine().begin() as conn:
        result =  conn.execute(samples.select() \
            .where(samples.c.project==project)) 
              
        df = pd.DataFrame(result.fetchall(), columns=result.keys())
        return df

     
def get_db_dict(db_field,request_param):
    db_ids_dict = {key: get_ids(request_param[key]) for key in db_field if key in request_param}
    # with get_db_session() as session:
    # get_db_value
    db_dict = { key:find_analyais_result_by_ids(value) for key,value in  db_ids_dict.items()}
    return db_dict

def parse_result(request_param,module_name):
    all_module = get_all_module("py_plot")
    if module_name not in all_module:
        raise HTTPException(status_code=500, detail=f"py_plot: {module_name}没有找到!")

    py_module = all_module[module_name]
    module = importlib.import_module(py_module)
    parse_data = getattr(module, "parse_data")
    sig = inspect.signature(parse_data)
    params = sig.parameters.keys()
    args = {
        "request_param":request_param,
    }
    if hasattr(module, "get_db_field"):
        get_db_field = getattr(module, "get_db_field")
        db_field = get_db_field()
        db_dict = get_db_dict(db_field,request_param)
        if "db_dict" in params:
            args.update({"db_dict":db_dict})
        else:
            args.update(db_dict)
    
    if "sample" in params:
        sample = get_sample(request_param['project'])
        args.update({"sample":sample})
    
    # data = None
    # if len(params) ==1:
    #     data = parse_data(request_param)
    # elif len(params) ==2:
    #     if hasattr(module, "get_db_field"):
    #         get_db_field = getattr(module, "get_db_field")
    #         db_field = get_db_field()
    #         db_dict = get_db_dict(db_field,request_param)
    #         data = parse_data(request_param,db_dict)
    #     else:
    #         sample = get_sample(request_param['project'])
    #         data = parse_data(request_param,sample)
    # else:
    #     get_db_field = getattr(module, "get_db_field")
    #     db_field = get_db_field()
    #     db_dict = get_db_dict(db_field,request_param)
    #     sample = get_sample(request_param['project'])
    #     data = parse_data(request_param,db_dict,sample)

    data = parse_data(**args)
    result = {}
    # if isinstance(data,list):

    if isinstance(data,dict):
        new_data = []
        for key, value in data.items():
            if not key.startswith("in_"):
                value = format_output(value,request_param)
                new_data.append(value)
        result= {"dataList":new_data}
    elif isinstance(data,tuple):
        new_data = []
        for item in data:
            if not isinstance(item,tuple) and not isinstance(item,list):
                if item is not None:
                    item = format_output(item,request_param)
                    new_data.append(item)
                # if isinstance(item, pd.DataFrame):
                #     new_data.append(item.to_dict(orient="records"))
                # else:
                #     if item:
                #         new_data.append(item)
        
        result = {"dataList":new_data}
    elif isinstance(data,list):
        pass
    else:
        new_data = format_output(data,request_param)
        result = {"dataList":[new_data]}
        # if isinstance(data, pd.DataFrame):
        #     # result = {"data":data.to_dict(orient="records")}
        #     result = {"data":json.loads(data.to_json(orient="records"))}
        # else:
        #     result = {"data":data}  
             
    if hasattr(module, "parse_plot"):
        parse_plot = getattr(module, "parse_plot")
        try:
            plt = parse_plot(data, request_param)
            if isinstance(plt,dict):
                for key, value in plt.items():
                    # buf = BytesIO()
                    # value.savefig(buf, format='png', bbox_inches='tight')
                    # value.close()  # 关闭图像以释放内存
                    # buf.seek(0)
                    # img_base64 = base64.b64encode(buf.read()).decode('utf-8')
                    img = format_img_output(value,request_param)
                    result.update({key:img })
            elif isinstance(plt,list):
                img_list = []
                for value in plt:
                    img = format_img_output(value,request_param)
                    img_list.append(img)
                result.update({"img": img_list})
                
            else:
                # buf = BytesIO()
                # plt.savefig(buf, format='png', bbox_inches='tight')
                # plt.close()  # 关闭图像以释放内存
                # buf.seek(0)
                # img_base64 = base64.b64encode(buf.read()).decode('utf-8')
                img = format_img_output(plt,request_param)
                result.update({"img":img})
        except Exception as e:
            # print("发生异常：",e.with_traceback())
            traceback.print_exc()
            raise HTTPException(status_code=500, detail=json.dumps(e.args))
    
    return result


def get_downstream_analysis_result_dir(request_param,suffix):
    settings = get_settings()
    base_dir = settings.BASE_DIR
    project,analysis_method = request_param['project'],request_param['analysis_method']
    
    downstream_analysis_result_dir  =   base_dir / project / "downstream_analysis_result" /analysis_method 
    if not downstream_analysis_result_dir.exists():
        downstream_analysis_result_dir.mkdir(parents=True,  exist_ok=True)
    str_uuid = str(uuid.uuid4())
    downstream_analysis_result = downstream_analysis_result_dir / f"{str_uuid}.{suffix}"
    file_name = str(downstream_analysis_result).replace(str(base_dir),"")
    return downstream_analysis_result,file_name


def format_img_output(plt,request_param):
    imgType = request_param['imgType']
    downstream_analysis_result,file_name = get_downstream_analysis_result_dir(request_param,imgType)
    plt.savefig(downstream_analysis_result, format='pdf', bbox_inches='tight')

    buf = BytesIO()
    plt.savefig(buf, format='png', bbox_inches='tight')
    plt.close()  # 关闭图像以释放内存
    buf.seek(0)
    img_base64 = base64.b64encode(buf.read()).decode('utf-8')
    img_base64 = "data:image/png;base64," + img_base64
    return {
        "data":img_base64,
        "type":"img",
        "url":f"/brave-api/dir{file_name}"
    }

# 如果请求中没有analysis_method 则不保存
def format_output(item,request_param):
    setting = get_settings()
    base_dir = str(setting.BASE_DIR)
    if  isinstance(item, pd.DataFrame):
        file_name = ""
        if "origin"   in request_param and "file_path" in request_param  :
            file_path = request_param['file_path']
            file_name = file_path.replace(base_dir,"/brave/brave-api/dir")
        else:     
            table_type = request_param['table_type']
            downstream_analysis_result,file_name = get_downstream_analysis_result_dir(request_param,table_type)
            
            if table_type=='tsv':
                item.to_csv(downstream_analysis_result, sep="\t", index=False)
            elif table_type=='xlsx':
                item.to_excel(downstream_analysis_result,  index=False)
        

        return  {
            "data":json.loads(item.to_json(orient="records")) ,
            "type":"table",
            "url":f"/brave-api/dir{file_name}"
        }
    elif isinstance(item, str):
        if item == 'html':
            file_path = request_param['file_path']
            file_name = file_path.replace(base_dir,"/brave/brave-api/dir")
            return {
                "data":file_name,
                "type":"html",
                "url":f"/brave-api/dir{file_name}"
            }
        return {
            "data":item,
            "type":"string",
        }
    else:
        return item


@file_parse_plot.get("/fast-api/file-parse-plot-test")
def parse_result_restful():
    # base_path ="/ssd1/wy/workspace2/test/test_workspace/result/V1.0"
    # verison = "V1.0"
    # project="test"
    file_path ="/ssd1/wy/workspace2/leipu/leipu_workspace2/output/prokka/OSP-3/OSP-3.txt"
    module_name = "prokka_txt_plot"
    result = parse_result(file_path, module_name)
    return result
# @file_parse_plot.post("/fast-api/file-parse-plot")
# async def parse_result_restful(request: Request):
#     # base_path ="/ssd1/wy/workspace2/test/test_workspace/result/V1.0"
#     # verison = "V1.0"
#     # project="test"
#     # file_path ="/ssd1/wy/workspace2/leipu/leipu_workspace2/output/prokka/OSP-3/OSP-3.txt"
#     # module_name = "prokka_txt_plot"
#     data = await request.json()
#     # result = parse_result(file_path, module_name)
#     return result

@file_parse_plot.post("/fast-api/file-parse-plot/{module_name}")
async def parse_result_restful(module_name,request_param: Dict[str, Any]):
    # base_path ="/ssd1/wy/workspace2/test/test_workspace/result/V1.0"
    # verison = "V1.0"
    # project="test"
    # file_path ="/ssd1/wy/workspace2/leipu/leipu_workspace2/output/prokka/OSP-3/OSP-3.txt"
    # module_name = "prokka_txt_plot"
    # data = await request.json()
    is_save_analysis_result = request_param['is_save_analysis_result']
    result = parse_result(request_param,module_name)
    if is_save_analysis_result:
        save_plot_result(result,request_param)
    return result

def save_plot_result(result, request_param):
    file_path = None
    if "id" in request_param:
        with get_db_session() as db:
            sampleAnalysisResult = db.query(SampleAnalysisResult) \
                .filter(SampleAnalysisResult.id == request_param["id"]).first()
        file_path =   sampleAnalysisResult.content
    else:
        file_path ,file_name= get_downstream_analysis_result_dir(request_param,"json")
        # str_uuid = str(uuid.uuid4())
        # file_path = f"/ssd1/wy/workspace2/nextflow-fastapi/analysis_result/{str_uuid}.json"

    
    with open(file_path, "w", encoding="utf-8") as f:
        json.dump(result, f, ensure_ascii=False, indent=4)
    software = request_param['software']
    project = request_param['project']
    analysis_method = request_param['analysis_method']
    analysis_name = request_param['analysis_name']
    new_analysis = {
        "request_param":json.dumps(request_param),
        "software":software ,
        "content_type":"file",
        "content":file_path,
        "project":project, 
        "analysis_method":analysis_method,
        "analysis_name":analysis_name,
        "analysis_type":"downstream"
    }
    with get_db_session() as db:
        if "id" in request_param:
            db.query(SampleAnalysisResult) \
                .filter(SampleAnalysisResult.id == request_param["id"]) \
                    .update(new_analysis)
        else:
            analysisResult = SampleAnalysisResult(**new_analysis)
            db.add(analysisResult)
        db.commit()
    

@file_parse_plot.post("/fast-api/file-save-parse-plot/{module_name}")
async def parse_result_restful(module_name,request_param: Dict[str, Any]):
    # base_path ="/ssd1/wy/workspace2/test/test_workspace/result/V1.0"
    # verison = "V1.0"
    # project="test"
    # file_path ="/ssd1/wy/workspace2/leipu/leipu_workspace2/output/prokka/OSP-3/OSP-3.txt"
    # module_name = "prokka_txt_plot"
    # data = await request.json()

    result = parse_result(request_param,module_name)
    save_plot_result(result)
    return result



# def update_or_save_result(request_param, software, content_type, content, db, project, verison, analysis_method,analysis_name):
#         # sampleAnalysisResult = db.query(SampleAnalysisResult) \
#         # .filter(and_(SampleAnalysisResult.analysis_method == analysis_method,\
#         #         SampleAnalysisResult.analysis_version == verison, \
#         #         SampleAnalysisResult.analysis_key == analysis_key, \
#         #         SampleAnalysisResult.project == project \
#         #     )).first()
#         # if sampleAnalysisResult:
#         #     sampleAnalysisResult.sample_name = sample_name
#         #     sampleAnalysisResult.content = content
#         #     sampleAnalysisResult.content_type = content_type
#         #     sampleAnalysisResult.analysis_name = analysis_name
#         #     # sampleAnalysisResult.log_path = log_path
#         #     sampleAnalysisResult.software = software
#         #     db.commit()
#         #     db.refresh(sampleAnalysisResult)
#         #     print(">>>>更新: ",sample_name, software, content_type)
#         # else:
#         sampleAnalysisResult = SampleAnalysisResult(analysis_method=analysis_method, \
#             analysis_version=verison, \
#             request_param=request_param, \
#             content_type=content_type, \
#             analysis_name=analysis_name, \
#             # log_path=log_path, \
#             software=software, \
#             project=project, \
#             content=content \
#                 )
#         db.add(sampleAnalysisResult)
#         db.commit()
#         print(">>>>新增: ",analysis_method, content_type)


@file_parse_plot.get("/fast-api/read-json")
async def read_json_restful(path):
    with open(path,"r") as f:
        res = json.load(f)
        return res


from sqlalchemy import Column, Table
from sqlalchemy.sql.sqltypes import Integer, String
from brave.api.config.db import meta
# from sqlalchemy.dialects.mysql import LONGTEXT
from sqlalchemy import Text

samples = Table(
    "t_samples",
    meta,
    Column("id", Integer, primary_key=True),
    Column("project", String(255)),
    Column("sample_key", String(255)),
    Column("sample_name", String(255)),
    Column("sequencing_target", String(255)),
    Column("sequencing_technique", String(255)),
    Column("sample_composition", String(255)),
    Column("library_name", String(255)),
    Column("sample_group", String(255)),
    Column("sample_group_name", String(255)),
    Column("sample_source", String(255)),
    Column("host_disease", String(255)),
    Column("sample_individual", String(255)),
    Column("is_available", Integer),
    Column("fastq1", String(255)),
    Column("fastq2", String(255)),
)
analysis = Table(
    "nextflow",
    meta,
    Column("id", Integer, primary_key=True),
    Column("project", String(255)),
    Column("analysis_key", String(255)),
    Column("analysis_name", String(255)),
    Column("input_file", String(255)),
    Column("analysis_method", String(255)),
    Column("work_dir", String(255)),
    Column("params_path", String(255)),
    Column("command_path", String(255)),
    Column("request_param", Text),
    Column("output_format", Text),
    Column("output_dir", String(255)),
    Column("pipeline_script", String(255)),
    Column("parse_analysis_module", String(255))
)

analysis_result = Table(
    "analysis_result",
    meta,
    Column("id", Integer, primary_key=True),
    Column("sample_name", String(255)),
    Column("sample_key", String(255)),
    Column("analysis_name", String(255)),
    Column("analysis_key", String(255)),
    Column("analysis_method", String(255)),
    Column("software", String(255)),
    Column("content", String(255)),
    Column("analysis_version", String(255)),
    Column("content_type", String(255)),
    Column("analysis_id", Integer),
    Column("project", String(255)),
    Column("request_param", String(255)),
    Column("analysis_type", String(255)),
    Column("create_date", String(255))
)

literature = Table(
    "literature",
    meta,
    Column("id", Integer, primary_key=True),
    Column("literature_key", String(255)),
    Column("literature_type", String(255)),
    Column("title", String(255)),
    Column("url", String(255)),
    Column("content", Text),
    Column("translate", Text),
    Column("interpretation", Text),
    Column("img", Text),
    Column("journal", String(255)),
    Column("publish_date", String(255)),
    Column("keywords", String(255))

)


relation_literature = Table(
    "relation_literature",
    meta,
    Column("relation_id", Integer, primary_key=True),
    Column("literature_key", String(255)),
    Column("obj_key", String(255)),
    Column("obj_type", String(255))
)
# meta.create_all(engine)


from typing import Optional
from pydantic import BaseModel

class AnalysisInput(BaseModel):
    id: Optional[int]= None
    project: Optional[str]
    samples: list
    analysis_method:str
    analysis_name:str
    
    # analysis_name: Optional[str]
    # work_dir: Optional[str]
    # output_dir: Optional[str]

class Analysis(BaseModel):
    id: Optional[int]
    project: Optional[str]
    analysis_key: Optional[str]
    analysis_method: Optional[str]
    analysis_name: Optional[str]
    input_file: Optional[str]
    request_param: Optional[str]
    work_dir: Optional[str]
    output_dir: Optional[str]
    params_path: Optional[str]
    output_format: Optional[str]
    command_path: Optional[str]
    pipeline_script: Optional[str]
    parse_analysis_module: Optional[str]


from sqlalchemy import Column, Integer, String
from brave.api.config.db import Base
# from sqlalchemy.dialects.mysql import LONGTEXT
from sqlalchemy import Text

class SampleAnalysisResult(Base):
    __tablename__ = "analysis_result"

    id = Column(Integer, primary_key=True, index=True)
    sample_name = Column(String(255))
    sample_key = Column(String(255))
    analysis_name = Column(String(255))
    analysis_key = Column(String(255))
    analysis_method = Column(String(255))
    software = Column(String(255))
    content = Column(Text)
    analysis_id = Column(Text)
    analysis_version = Column(String(255))
    content_type = Column(String(255))
    project = Column(String(255))
    request_param = Column(Text)
    analysis_type = Column(String(255))
    create_date = Column(String(255))


    # def __repr__(self):
    #     return user_to_dict(self)


# class Sample(Base):
#     __tablename__ = "t_samples"

#     id = Column(Integer, primary_key=True, index=True)
#     sample_name = Column(String(255))
#     sample_key = Column(String(255))
#     sample_group= Column(String(255))

# Base.metadata.create_all(bind=engine)



from fastapi import APIRouter
import pandas as pd
import json

bio_database = APIRouter()

@bio_database.get("/fast-api/get_metaphlan_clade")
def get_metaphlan_clade():
    with open("/ssd1/wy/workspace2/nextflow-fastapi/databases/clade.json") as f:
        data = json.load(f)
    return data


from typing import Optional
from pydantic import BaseModel

class AnalysisResultQuery(BaseModel):
    # id: Optional[int]
    analysis_method:Optional[list]=None
    project:Optional[str]=None
    querySample:Optional[bool]=True
    analysis_type:Optional[str]=None
    ids:Optional[list]=None
    

class AnalysisResult(BaseModel):
    id: Optional[int]
    sample_name: Optional[str]
    sample_key: Optional[str]
    analysis_name: Optional[str]
    analysis_key: Optional[str]
    analysis_method: Optional[str]
    software: Optional[str]
    content: Optional[str]
    analysis_version: Optional[str]
    content_type: Optional[str]
    project: Optional[str]
    request_param: Optional[str]
    analysis_id: Optional[int]
    sample_group: Optional[str]
    sample_group_name: Optional[str]
    analysis_type: Optional[str]
    create_date: Optional[str]
    sample_source: Optional[str]
    host_disease: Optional[str]
    


from fastapi import APIRouter,Depends
from sqlalchemy.orm import Session

# from brave.api.config.db import conn
# from models.user import users
from typing import List
from starlette.status import HTTP_204_NO_CONTENT
from sqlalchemy import func, select
from brave.api.models.orm import SampleAnalysisResult
import glob
import importlib
import os
import json
from brave.api.config.db import get_db_session
from sqlalchemy import and_,or_
from brave.api.schemas.analysis_result import AnalysisResultQuery,AnalysisResult
from brave.api.models.core import samples,analysis_result
from brave.api.config.db import get_engine
import inspect
from fastapi import HTTPException
from brave.api.service.pipeline  import get_all_module,get_pipeline_dir


sample_result = APIRouter()
# key = Fernet.generate_key()
# f = Fernet(key)


# def get_all_subclasses(cls):
#     subclasses = set(cls.__subclasses__())
#     for subclass in subclasses.copy():
#         subclasses.update(get_all_subclasses(subclass))
#     return subclasses




def update_or_save_result(analysis_key,sample_name, software, content_type, content, db, project, verison, analysis_method,analysis_name,analysis_id):
        sampleAnalysisResult = db.query(SampleAnalysisResult) \
        .filter(and_(SampleAnalysisResult.analysis_method == analysis_method,\
                SampleAnalysisResult.analysis_version == verison, \
                SampleAnalysisResult.analysis_key == analysis_key, \
                SampleAnalysisResult.project == project \
            )).first()
        if sampleAnalysisResult:
            sampleAnalysisResult.sample_name = sample_name
            sampleAnalysisResult.content = content
            sampleAnalysisResult.sample_key=sample_name
            sampleAnalysisResult.content_type = content_type
            sampleAnalysisResult.analysis_name = analysis_name
            sampleAnalysisResult.analysis_id = analysis_id
            sampleAnalysisResult.analysis_type="upstream"
            # sampleAnalysisResult.log_path = log_path
            sampleAnalysisResult.software = software
            db.commit()
            db.refresh(sampleAnalysisResult)
            print(">>>>更新: ",sample_name, software, content_type)
        else:
            sampleAnalysisResult = SampleAnalysisResult(analysis_method=analysis_method, \
                analysis_version=verison, \
                sample_name=sample_name, \
                content_type=content_type, \
                analysis_name=analysis_name, \
                analysis_key=analysis_key, \
                analysis_id=analysis_id, \
                analysis_type="upstream", \
                # log_path=log_path, \
                software=software, \
                project=project, \
                sample_key=sample_name, \
                content=content \
                    )
            db.add(sampleAnalysisResult)
            db.commit()
            print(">>>>新增: ",sample_name, software, content_type)

def parse_result_oneV2(analysis_method,module_args,analsyisRecord,module,dir_path,project,verison,analysis_id=-1):
    parse = getattr(module, "parse")
    sig = inspect.signature(parse)
    params = sig.parameters
    res = None

    # if len(params)==1:
    #     res = parse(dir_path)
    # elif len(params)==2:
    #     res = parse(dir_path,analsyisRecord)
    # elif len(params) ==3:
    
    args = {
        "dir_path":dir_path,
        "analysis": analsyisRecord,
        **module_args
    }
    res = parse(**args)

    if hasattr(module,"get_analysis_method"):
        get_analysis_method = getattr(module, "get_analysis_method")
        analysis_method = get_analysis_method()
        print(f">>>>>更改分析名称: {analysis_method}")
    analysis_name = analysis_method
    if hasattr(module,"get_analysis_name"):
        get_analysis_name = getattr(module, "get_analysis_name")
        analysis_name = get_analysis_name()
        
    with get_db_session() as db:
        if len(res) >0:
            # print(res[0])
            if len(res[0]) == 4:
                for analysis_key,software,content_type,content in res:
                    update_or_save_result(analysis_key,analysis_key, software, content_type, content, db, project, verison, analysis_method,analysis_name,analysis_id)
            elif len(res[0]) == 5:
                for analysis_key,sample_name,software,content_type,content in res:
                    update_or_save_result(analysis_key,sample_name, software, content_type, content, db, project, verison, analysis_method,analysis_name,analysis_id)


def parse_result_one(analysis_method,module,dir_path,project,verison,analysis_id=-1):
    parse = getattr(module, "parse")
    res = parse(dir_path)
    if hasattr(module,"get_analysis_method"):
        get_analysis_method = getattr(module, "get_analysis_method")
        analysis_method = get_analysis_method()
        print(f">>>>>更改分析名称: {analysis_method}")
    analysis_name = analysis_method
    if hasattr(module,"get_analysis_name"):
        get_analysis_name = getattr(module, "get_analysis_name")
        analysis_name = get_analysis_name()
        
    with get_db_session() as db:
        if len(res) >0:
            # print(res[0])
            if len(res[0]) == 4:
                for analysis_key,software,content_type,content in res:
                    update_or_save_result(analysis_key,analysis_key, software, content_type, content, db, project, verison, analysis_method,analysis_name,analysis_id)
            elif len(res[0]) == 5:
                for analysis_key,sample_name,software,content_type,content in res:
                    update_or_save_result(analysis_key,sample_name, software, content_type, content, db, project, verison, analysis_method,analysis_name,analysis_id)
            # print(sample_name)

def parse_result(dir_path,project,verison):
    # pipeline_dir = get_pipeline_dir()
    # py_files = [f for f in os.listdir(f"{pipeline_dir}/*/py_sample_result_parse/*") if f.endswith('.py')]
    # py_files = glob.glob(f"{pipeline_dir}/*/py_sample_result_parse/*.py")
    py_files = get_all_module("py_sample_result_parse")
    for key,py_module in py_files.items():
        # module_name = py_file[:-3]  # 去掉 `.py` 后缀，获取模块名

       
        # if module_name not in all_module:
        #     raise HTTPException(status_code=500, detail=f"py_sample_result_parse: {module_name}没有找到!")
        # py_module = all_module[module_name]
        module = importlib.import_module(py_module)
        support_analysis_method = getattr(module, "support_analysis_method")
        analysis_method = support_analysis_method()

        
        if dir_path.endswith(analysis_method):
            print(f">>>>>找到分析名称 {analysis_method} 的分析结果")
            parse_result_one(analysis_method,module,dir_path,project,verison)
            # parse = getattr(module, "parse")
            # res = parse(dir_path)
            # if hasattr(module,"get_analysis_method"):
            #     get_analysis_method = getattr(module, "get_analysis_method")
            #     analysis_method = get_analysis_method()
            #     print(f">>>>>更改分析名称: {analysis_method}")
            # analysis_name = analysis_method
            # if hasattr(module,"get_analysis_name"):
            #     get_analysis_name = getattr(module, "get_analysis_name")
            #     analysis_name = get_analysis_name()
             
            # with get_db_session() as db:
            #     if len(res) >0:
            #         # print(res[0])
            #         if len(res[0]) == 4:
            #             for analysis_key,software,content_type,content in res:
            #                 update_or_save_result(analysis_key,analysis_key, software, content_type, content, db, project, verison, analysis_method,analysis_name)
            #         elif len(res[0]) == 5:
            #              for analysis_key,sample_name,software,content_type,content in res:
            #                 update_or_save_result(analysis_key,sample_name, software, content_type, content, db, project, verison, analysis_method,analysis_name)
            #         # print(sample_name)


@sample_result.get("/sample-parse-result-test-hexiaoyan",tags=['analsyis_result'])
def parse_result_restful():
    # base_path ="/ssd1/wy/workspace2/test/test_workspace/result/V1.0"
    # verison = "V1.0"
    # project="test"
    base_path ="/ssd1/wy/workspace2/hexiaoyan/hexiaoyan_workspace2/output"
    verison = "V1.0"
    project="hexiaoyan"
    
    dir_list = glob.glob(f"{base_path}/*",recursive=True)
    for dir_path in dir_list:
        parse_result(dir_path,project,verison)
    return {"msg":"success"}

@sample_result.get("/sample-parse-result-test",tags=['analsyis_result'])
def parse_result_restful():
    # base_path ="/ssd1/wy/workspace2/test/test_workspace/result/V1.0"
    # verison = "V1.0"
    # project="test"
    base_path ="/ssd1/wy/workspace2/leipu/leipu_workspace2/output"
    verison = "V1.0"
    project="leipu"
    
    dir_list = glob.glob(f"{base_path}/*",recursive=True)
    for dir_path in dir_list:
        parse_result(dir_path,project,verison)
    return {"msg":"success"}

@sample_result.get("/sample-parse-result-test-leipu-meta",tags=['analsyis_result'])
def parse_result_restful():
    # base_path ="/ssd1/wy/workspace2/test/test_workspace/result/V1.0"
    # verison = "V1.0"
    # project="test"
    base_path ="/ssd1/wy/workspace2/leipu/leipu_workspace_meta/output"
    verison = "V1.0"
    project="leipu"
    
    dir_list = glob.glob(f"{base_path}/*",recursive=True)
    for dir_path in dir_list:
        parse_result(dir_path,project,verison)
    return {"msg":"success"}

@sample_result.get("/sample-parse-result")
def parse_result_restful(base_path,verison,project):
    # base_path ="/ssd1/wy/workspace2/test/test_workspace/result"
    # verison = "V1.0"
    
    dir_list = glob.glob(f"{base_path}/{verison}/*",recursive=True)
    for dir_path in dir_list:
        parse_result(dir_path,project,verison)
    return {"msg":"success"}

def find_analyais_result_by_ids( value):
    ids = value
    if not isinstance(value,list):
        ids = [value]
    analysis_result = find_analyais_result(AnalysisResultQuery(ids=ids))

    # analysis_result =  session.query(SampleAnalysisResult) \
    #             .filter(SampleAnalysisResult.id.in_(ids)) \
    #                 .all()
                    
    # for item in analysis_result:
    #     if item.content_type=="json" and not isinstance(item.content, dict):
    #         item.content = json.loads(item.content)

    if len(analysis_result)!=len(ids):
        raise HTTPException(status_code=500, detail="数据存在问题!")
    if not isinstance(value,list) and len(analysis_result)==1:
        return analysis_result[0]
    else:
        return analysis_result
    

def find_analyais_result(analysisResultQuery:AnalysisResultQuery):
    return find_analyais_result_by_analysis_method(analysisResultQuery)

@sample_result.post(
    "/fast-api/find-analyais-result-by-analysis-method",
    response_model=List[AnalysisResult])
def find_analyais_result_by_analysis_method(analysisResultQuery:AnalysisResultQuery):
    # with get_db_session() as session:
    #     analysis_result =  session.query(SampleAnalysisResult,Sample) \
    #         .outerjoin(Sample, SampleAnalysisResult.sample_key == Sample.sample_key) \
    #         .filter(and_( \
    #             SampleAnalysisResult.analysis_method.in_(analysisResultQuery.analysis_method), \
    #             SampleAnalysisResult.project == analysisResultQuery.project \
    #         )) \
    #             .all()

    #     for item in analysis_result:
    #         if item.content_type=="json":
    #             item.content = json.loads(item.content)
            # print()
    with get_engine().begin() as conn:
        stmt = analysis_result.select() 
        if analysisResultQuery.querySample:
            stmt =  select(analysis_result, samples).select_from(analysis_result.outerjoin(samples,samples.c.sample_key==analysis_result.c.sample_key))
        
        conditions = []
        if analysisResultQuery.project is not None:
            conditions.append(analysis_result.c.project == analysisResultQuery.project)
        if analysisResultQuery.ids is not None:
            conditions.append(analysis_result.c.id.in_(analysisResultQuery.ids))
        if analysisResultQuery.analysis_method is not None:
            conditions.append(analysis_result.c.analysis_method.in_(analysisResultQuery.analysis_method))
        if analysisResultQuery.analysis_type is not None:
            conditions.append(analysis_result.c.analysis_type == analysisResultQuery.analysis_type)
        stmt= stmt.where(and_( *conditions))
        
        result  = conn.execute(stmt)
        # result = result.fetchall()
        rows = result.mappings().all()
        result_dict = [AnalysisResult(**row) for row in rows]
        # result_dict = []
        for item in result_dict:
            if item.content_type=="json" and not isinstance(item.content, dict):
                item.content = json.loads(item.content)
        #     result.append(row)
        # # rows = result.mappings().all()
        # pass
    return result_dict
    # return {"aa":"aa"}


@sample_result.delete("/analyais-result/delete-by-id/{id}",  status_code=HTTP_204_NO_CONTENT)
def delete_analysis_result(id: int):
    with get_engine().begin() as conn:
        conn.execute(analysis_result.delete().where(analysis_result.c.id == id))
    return {"message":"success"}



@sample_result.post(
    "/fast-api/add-sample-analysis",
    tags=['analsyis_result'],
    description="根据项目名称导入样本")
def add_sample_analysis(project):
    insert_sample_list = []
    with get_engine().begin() as conn:
        stmt = samples.select().where(samples.c.project==project)
        result  = conn.execute(stmt).fetchall()
        sample_list = [dict(row._mapping) for row in result]
        # sample_list = sample_list[1:2]
        
        for item in sample_list:
            insert_sample_list.append({
                "sample_key":item['sample_key'],
                "sample_name":item['sample_name'],
                "analysis_method":f"V1_{item['sample_composition']}_{item['sequencing_technique']}_{item['sequencing_target']}",
                "content":json.dumps({
                    "fastq1":item['fastq1'],
                    "fastq2":item['fastq2']
                }),
                "project":item['project'],
                "content_type":"json",
            })
    with get_db_session() as db:
        for item in insert_sample_list:
            analysisResult = db.query(SampleAnalysisResult) \
                    .filter(and_(SampleAnalysisResult.sample_key == item['sample_key'], \
                        SampleAnalysisResult.analysis_method == item['analysis_method'] \
                        )).first()
            if analysisResult:
                analysisResult.sample_name = item['sample_name']
                analysisResult.analysis_method = item['analysis_method']
                analysisResult.content = item['content']
                analysisResult.project = item['project']
                analysisResult.content_type = item['content_type']
            else:
                analysisResult = SampleAnalysisResult(**item)
                db.add(analysisResult)
            db.commit()
            db.refresh(analysisResult)

        return {"message":"success"}





import pandas as pd
from functools import reduce
from itertools import chain
def get_last_num(clade_name,num):
    clade_name_list = clade_name.split('|')
    return clade_name_list[len(clade_name_list)-num]

def get_last(clade_name):
    clade_name_list = clade_name.split('|')
    return clade_name_list[len(clade_name_list)-1]
def set_rank(taxonomy):
    if taxonomy.startswith("t__"):
        return "SGB"
    if taxonomy.startswith("s__"):
        return "SPECIES"
    elif  taxonomy.startswith("g__"):
        return "GENUS"
    elif  taxonomy.startswith("f__"):
        return "FAMILY"
    elif  taxonomy.startswith("o__"):
        return "ORDER"
    elif  taxonomy.startswith("c__"):
        return "CLASS"
    elif  taxonomy.startswith("p__"):
        return "PHYLUM"
    elif  taxonomy.startswith("k__"):
        return "KINGDOM"
def rename_taxonomy(taxonomy):
    if taxonomy.startswith("t__"):
        return taxonomy.replace("t__","").replace("_"," ")
    if taxonomy.startswith("s__"):
        return taxonomy.replace("s__","").replace("_"," ")
    elif  taxonomy.startswith("g__"):
        return taxonomy.replace("g__","").replace("_"," ")
    elif  taxonomy.startswith("f__"):
        return taxonomy.replace("f__","").replace("_"," ")
    elif  taxonomy.startswith("o__"):
        return taxonomy.replace("o__","").replace("_"," ")
    elif  taxonomy.startswith("c__"):
        return taxonomy.replace("c__","").replace("_"," ")
    elif  taxonomy.startswith("p__"):
        return taxonomy.replace("p__","").replace("_"," ")
    elif  taxonomy.startswith("k__"):
        return taxonomy.replace("k__","").replace("_"," ")
def get_one_df(file,sample_key):
    df = pd.read_csv(file,sep="\t",comment="#",header=None)
    df.columns = ["clade_name","ncbi_tax_id","relative_abundance","additional_species"]
    df = df.rename({"relative_abundance":sample_key},axis=1)
    df = df.drop("additional_species",axis=1)
    df = df.set_index(["clade_name","ncbi_tax_id"])
    return df

def get_abundance(metaphlan_sam_abundance):
    # metaphlan_sam_abundance = db_dict['metaphlan_sam_abundance']
    df_list = [get_one_df(item.content['profile'],item.sample_key) for item in metaphlan_sam_abundance]
    df = reduce(lambda x,y:pd.merge(x,y,left_index=True,right_index=True, how="outer"),df_list)
    df = df.reset_index()
    df['taxonomy'] = df.apply(lambda x : get_last(x['clade_name']) ,axis=1)
    df['tax_id'] = df.apply(lambda x : get_last(x['ncbi_tax_id']) ,axis=1)
    df['rank'] = df.apply(lambda x : set_rank(x['taxonomy']) ,axis=1)
    df['taxonomy'] = df.apply(lambda x : rename_taxonomy(x['taxonomy']) ,axis=1)
    df = df.set_index(["clade_name","ncbi_tax_id","taxonomy","tax_id","rank"])
    df = df.fillna(0)
    
    # df.to_pickle("test/test.pkl")

    return df

# def get_metadata(control_sample,treatment_sample,control_group,treatment_group):
#     control_meta = pd.DataFrame([(item.sample_key,control_group) for item in control_sample],columns=['sample_key','group'])
#     treatment_meta = pd.DataFrame([(item.sample_key,treatment_group) for item in treatment_sample],columns=['sample_key','group'])
#     metadata = pd.concat([control_meta, treatment_meta], ignore_index=True)
#     metadata = metadata.set_index("sample_key")
#     return metadata

def get_metadata_group(db_dict,group,group_name):
    return pd.DataFrame([(item.sample_key,group_name) for item in db_dict[group] ],columns=['sample_key','group'])

def get_metadata(db_dict,groups):
   df_list = [get_metadata_group(db_dict,group,group_name) for group,group_name in groups.items()]
   metadata = pd.concat(df_list, ignore_index=True)
   metadata = metadata.set_index("sample_key")
   return metadata



def get_abundance_metadata(request_param,db_dict,groups):
    samples = sum([db_dict[group] for group in groups],[])
    abundance = get_abundance(samples)
    groups = {group:"-".join(request_param[group]['group']) for group in groups}
    # control_sample = db_dict['control']
    # treatment_sample = db_dict['treatment']
    # control_group = "-".join(request_param['control']['group'])
    # treatment_group = "-".join(request_param['treatment']['group'])
    metadata = get_metadata(db_dict,groups)

    # metadata = get_metadata(control_sample,treatment_sample,control_group,treatment_group)
    # abundance1 = abundance.reset_index(["clade_name","taxonomy","tax_id","rank"])[["clade_name","taxonomy","tax_id","rank"]].reset_index(drop=True)
    rank = "SPECIES"
    if "rank" in request_param:
        rank = request_param['rank']
    abundance = abundance.reset_index(['taxonomy','rank']).reset_index(drop=True).query("rank==@rank").drop("rank",axis=1).set_index("taxonomy").T
    return abundance,metadata,groups


def get_abundance_metadata4(request_param,db_dict,groups):
    samples = sum([db_dict[group] for group in groups],[])
    abundance = get_abundance(samples)
    groups = {group:"-".join(request_param[group]['group']) for group in groups}
    # control_sample = db_dict['control']
    # treatment_sample = db_dict['treatment']
    # control_group = "-".join(request_param['control']['group'])
    # treatment_group = "-".join(request_param['treatment']['group'])
    metadata = get_metadata(db_dict,groups)

    # metadata = get_metadata(control_sample,treatment_sample,control_group,treatment_group)
    abundance1 = abundance.reset_index(["clade_name","taxonomy","tax_id","rank"])[["clade_name","taxonomy","tax_id","rank"]].reset_index(drop=True)
    rank = "SPECIES"
    if "rank" in request_param:
        rank = request_param['rank']
    abundance = abundance.reset_index(['taxonomy','rank']).reset_index(drop=True).query("rank==@rank").drop("rank",axis=1).set_index("taxonomy").T
    return abundance,metadata,groups,abundance1


from fastapi import APIRouter,Depends
from sqlalchemy.orm import Session
# from brave.api.config.db import conn
from brave.api.schemas.sample import Sample,SampleGroup,SampleGroupQuery,ImportSample,Sample,ProjectCount
from typing import List
from starlette.status import HTTP_204_NO_CONTENT
from sqlalchemy import func, select
from brave.api.models.orm import SampleAnalysisResult
import glob
import importlib
import os
from brave.api.config.db import get_db_session
from sqlalchemy import and_,or_
import pandas as pd
from brave.api.models.core import samples
from brave.api.config.db import get_engine
from io import StringIO
from fastapi import HTTPException


sample = APIRouter()


def update_or_save_result(db,project,sample_name,file_type,file_path,log_path,verison,analysis_name,software):
        sampleAnalysisResult = db.query(SampleAnalysisResult) \
        .filter(and_(SampleAnalysisResult.analysis_name == analysis_name,\
                SampleAnalysisResult.analysis_verison == verison, \
                SampleAnalysisResult.sample_name == sample_name, \
                SampleAnalysisResult.file_type == file_type, \
                SampleAnalysisResult.project == project \
            )).first()
        if sampleAnalysisResult:
            sampleAnalysisResult.contant_path = file_path
            sampleAnalysisResult.log_path = log_path
            sampleAnalysisResult.software = software
            db.commit()
            db.refresh(sampleAnalysisResult)
            print(">>>>更新: ",file_path,sample_name,file_type,log_path)
        else:
            sampleAnalysisResult = SampleAnalysisResult(analysis_name=analysis_name, \
                analysis_verison=verison, \
                sample_name=sample_name, \
                file_type=file_type, \
                log_path=log_path, \
                software=software, \
                project=project, \
                contant_path=file_path \
                    )
            db.add(sampleAnalysisResult)
            db.commit()
            print(">>>>新增: ",file_path,sample_name,file_type,log_path)


@sample.post("/fast-api/import_sample_form_str",tags=["sample"],)
def import_sample_form_str(sample:ImportSample):
    csv_buffer = StringIO(sample.content)
    df = pd.read_csv(csv_buffer)
    # pass
    # df = pd.read_csv(path)
    with get_engine().begin() as conn:
        df_dict = df.to_dict(orient="records")
        stmt = samples.insert().values(df_dict)
        conn.execute(stmt)
        # print()
        # return conn.execute(samples.select()).fetchall()
        # print()
        return {"msg":"success"}


@sample.post("/fast-api/update_sample_form_str",tags=["sample"],)
def import_sample_form_str(sample:ImportSample):
    csv_buffer = StringIO(sample.content)
    df = pd.read_csv(csv_buffer)
    if not "project" in df.columns or not "sample_name" in df.columns:
        raise HTTPException(status_code=500, detail=f"必须包含project和sample_name") 
    with get_engine().begin() as conn:
        df_dict = df.to_dict(orient="records")
        for item in df_dict:
            stmt = samples.update().values(item).where(and_(
                samples.c.project==item['project'],
                samples.c.sample_name==item['sample_name'],
            ) )
            conn.execute(stmt)
        # print()
        # return conn.execute(samples.select()).fetchall()
        # print()
    return {"msg":"success"}

# ,response_model=List[Sample]
# /ssd1/wy/workspace2/leipu/leipu_workspace2/sample/RNA.sample.csv
@sample.get("/import-sample",tags=["sample"],)
def import_sample(path):
    df = pd.read_csv(path)
    with get_engine().begin() as conn:
        df_dict = df.to_dict(orient="records")
        stmt = samples.insert().values(df_dict)
        conn.execute(stmt)
        # print()
        # return conn.execute(samples.select()).fetchall()
        # print()
        return {"msg":"success"}

@sample.get("/update-import-sample",tags=["sample"],description="更新样本")
def import_sample(path):
    df = pd.read_csv(path)
    with get_engine().begin() as conn:
        df_dict = df.to_dict(orient="records")
        for item in df_dict:
            stmt = samples.update().values(item).where(samples.c.library_name==item['library_name'])
            conn.execute(stmt)
        # print()
        # return conn.execute(samples.select()).fetchall()
        # print()
        return {"msg":"success"}

@sample.post(
    "/fast-api/find-sample",
    response_model=List[SampleGroup],
    tags=["sample"],
    description="查找样本"
)
def get_analysis(query:SampleGroupQuery):
    with get_engine().begin() as conn:
        return conn.execute(samples.select() \
            .where(and_(samples.c.project==  query.project \
                ,samples.c.sample_composition==  query.sample_composition \
                ,samples.c.sequencing_target==  query.sequencing_target \
                ,samples.c.sequencing_technique==  query.sequencing_technique \
                )) \
                ).fetchall()

@sample.get(
    "/list-by-project",
    tags=["sample"],
    response_model=List[Sample] )
def list_by_project(project):
    with get_engine().begin() as conn:
        return conn.execute(samples.select() \
            .where(and_(samples.c.project==  project )) ).fetchall()

@sample.get(
    "/list-project",
    tags=['sample'],
    response_model=List[ProjectCount],
    description="列出所有项目")
def list_project():
    with get_engine().begin() as conn:
        stmt = (
            select(
                samples.c.project,
                func.count(samples.c.id).label("count")
            )
            .group_by(samples.c.project)
        )
        result = conn.execute(stmt).fetchall()
        # data = [dict(row._mapping) for row in result.fetchall()]
        return result


from brave.main import create_app
import uvicorn
import typer
import os
from brave.api.config.db import init_engine
# from brave.api.config.config import get_settings
from brave.api.config.db import meta,Base

app = typer.Typer()


@app.command()
def main(
    host: str = typer.Option("0.0.0.0", help="Host to bind"), 
    port: int =  typer.Option(5000, help="Port to bind"),
    reload: bool =  typer.Option(False, help="reload"),
    base_dir: str =typer.Option(None, help="Base directory"),
    work_dir: str =typer.Option(None, help="Work directory"),
    pipeline_dir: str =typer.Option(None, help="Pipeline directory"),
    literature_dir: str =typer.Option(None, help="Literature directory"),
    db_type: str =typer.Option("sqlite", help="Db type[ mysql, sqlite ]"),
    mysql_url: str =typer.Option("root:123456@192.168.3.60:53306/pipeline", help="Mysql url")
    ):
    
    
    os.environ["DB_TYPE"] = db_type
    os.environ["MYSQL_URL"] = mysql_url
    if base_dir:
        os.environ["BASE_DIR"] = base_dir
    if work_dir:
        os.environ["WORK_DIR"] = work_dir
    if pipeline_dir:
        os.environ["PIPELINE_DIR"] = pipeline_dir
    if literature_dir:
        os.environ["LITERATURE_DIR"] = literature_dir



    # settings = get_settings()
    engine = init_engine()

    meta.create_all(engine)
    Base.metadata.create_all(bind=engine)

    # typer.echo(f"base_dir={base_dir}, host={host}, port={port}")
    uvicorn.run("brave.main:create_app", host=host, port=port, factory=True,reload=reload)


if __name__ == "__main__":
    app()


from typing import Optional
from pydantic import BaseModel

class Literature(BaseModel):
    id: Optional[int]
    title: Optional[str]
    url: Optional[str]
    content: Optional[str]
    relation_id: Optional[int]=None
    translate: Optional[str]=None
    interpretation: Optional[str]=None
    img: Optional[str]=None
    journal: Optional[str]=None
    publish_date: Optional[str]=None
    keywords: Optional[str]=None
    literature_key: Optional[str]=None
    literature_type: Optional[str]=None


  

class LiteratureQuery(BaseModel):
    title: Optional[str]=None
    url: Optional[str]=None
    content: Optional[str]=None
    obj_key: Optional[str]=None
    obj_type: Optional[str]=None
    page_number: Optional[int] = 1
    page_size: Optional[int] = 10
    literature_key: Optional[str]=None


from fastapi import APIRouter
from brave.api.config.db import get_engine
from brave.api.models.core import literature,relation_literature
from brave.api.schemas.literature import Literature,LiteratureQuery
from sqlalchemy import select, and_, join, func,insert,update
from typing import List, Optional, Dict, Any
import json
import glob
import os
from brave.api.config.config import get_settings
literature_api = APIRouter()



@literature_api.post(
    "/literature",
    tags=['literature']
)
def get_analysis(query: LiteratureQuery) -> Dict[str, Any]:
    with get_engine().begin() as conn:
        offset = (query.page_number - 1) * query.page_size

        if query.obj_key and query.obj_type:
            # JOIN 查询
            j = join(
                literature,
                relation_literature,
                literature.c.literature_key == relation_literature.c.literature_key
            )
            base_stmt = select(literature,relation_literature).select_from(j).where(
                and_(
                    relation_literature.c.obj_key == query.obj_key,
                    relation_literature.c.obj_type == query.obj_type
                )
            )
            count_stmt = select(func.count()).select_from(j).where(
                and_(
                    relation_literature.c.obj_key == query.obj_key,
                    relation_literature.c.obj_type == query.obj_type
                )
            )
        else:
            # 单表查询
            base_stmt = select(literature)
            count_stmt = select(func.count()).select_from(literature)

        # 执行分页
        paged_stmt = base_stmt.offset(offset).limit(query.page_size)
        rows = conn.execute(paged_stmt).mappings().all()
        total = conn.execute(count_stmt).scalar()
        items = [Literature(**row) for row in rows]
        for item in items:
            if item.img:
                item.img = f"/brave-api/literature/dir/{item.img}"
            if item.keywords:
                item.keywords = json.loads(item.keywords)
        return {
            "items":items,
            "total": total,
            "page_number":query.page_number,
            "page_size":query.page_size
        }

def get_literature_one(item):
    with open(item,"r") as f:
        value = json.load(f)
    return value

def add_literature_key(item,literature_key):
    item.update({"literature_key":literature_key})
    return item

def add_or_update_relation_literature(conn,literature_key,literature_relation_data):
    # new_data_list = [add_literature_key(item,literature_key) for item in literature_relation_data]
    for item in literature_relation_data:
        new_data = {
            **item,
            "literature_key":literature_key
        }
        conditation = and_(relation_literature.c.obj_key == item["obj_key"],
                    relation_literature.c.literature_key == literature_key)
        stmt = select(relation_literature).where(conditation)
        result = conn.execute(stmt).fetchone()
        if result:
            update_stmt = (
                update(relation_literature)
                .where(conditation)
                .values(**new_data)
            )
            conn.execute(update_stmt)
        else:
            insert_stmt = insert(relation_literature).values(**new_data)
            conn.execute(insert_stmt)
            
    

@literature_api.post(
    "/literature/import",
    tags=['literature']
)
def import_literature():
    setting = get_settings()
    literature_dir = setting.LITERATURE_DIR
    literature_file_list = glob.glob(f"{literature_dir}/json/*.json")
    literature_dict = {os.path.basename(item).replace(".json",""):get_literature_one(item) for item in literature_file_list}
    for key,literature_list in literature_dict.items():
        with get_engine().begin() as conn:
            for literatureItem in literature_list:
                new_data = {k: v for k, v in literatureItem.items() if k != "relation"}
                new_data['keywords'] = json.dumps(new_data['keywords'])
                literature_relation_data = literatureItem["relation"]
                add_or_update_relation_literature(conn,literatureItem["literature_key"],literature_relation_data)

                stmt = select(literature.c.literature_key).where(
                    literature.c.literature_key == literatureItem["literature_key"]
                )
                result = conn.execute(stmt).fetchone()
                if result:
                    update_stmt = (
                        update(literature)
                        .where(literature.c.literature_key == literatureItem["literature_key"])
                        .values(**new_data)
                    )
                    conn.execute(update_stmt)
                else:
                    insert_stmt = insert(literature).values(**new_data)
                    conn.execute(insert_stmt)
            


from fastapi import APIRouter
from importlib.resources import files, as_file

import json
import os
import glob
from brave.api.config.config import get_settings
from brave.api.service.pipeline import get_pipeline_dir,get_pipeline_list
from collections import defaultdict

pipeline = APIRouter()
# BASE_DIR = os.path.dirname(__file__)

@pipeline.get("/get-pipeline/{name}",tags=['pipeline'])
async def get_pipeline(name):
    pipeline_dir =  get_pipeline_dir()
    # filename = f"{name}.json"
    json_file = f"{pipeline_dir}/{name}/main.json"
    data = {
        "files":json_file,
        # "wrapAnalysisPipeline":name,
        "exists":os.path.exists(json_file)
    }
    if os.path.exists(json_file):
        with open(json_file,"r") as f:
            json_data = json.load(f)
            data.update(json_data)
    return data


def get_config():
    pipeline_dir =  get_pipeline_dir()
    config = f"{pipeline_dir}/config.json"
    if os.path.exists(config):
        with open(config,"r") as f:
            return json.load(f)
    else:
        return {}
    
def get_pipeline_one(item):
    with open(item,"r") as f:
        data = json.load(f)
    data = {
        "path":os.path.basename(os.path.dirname(item)),
        "name":data['name'],
        "category":data['category'],
        "img":f"/brave-api/img/{data['img']}",
        "tags":data['tags'],
        "description":data['description'],
        "order":data['order']
    }
    return data

def get_category(name,key):
    config = get_config()
    if "category" in config:
        category = config['category']
        if name in category:
            return category[name][key]
    return name



@pipeline.get("/list-pipeline",tags=['pipeline'])
async def get_pipeline():
    # json_file = str(files("brave.pipeline.config").joinpath("config.json"))
    # with open(json_file,"r") as f:
    #     config = json.load(f)
    # pipeline_files = files("brave.pipeline")
    pipeline_files = get_pipeline_list()
    pipeline_files = [get_pipeline_one(str(item)) for item in pipeline_files]
    pipeline_files = sorted(pipeline_files, key=lambda x: x["order"])
    grouped = defaultdict(list)
    for item in pipeline_files:
        grouped[item["category"]].append(item)

    result = []


    for category, items in grouped.items():
        result.append({
            "name": get_category(category,"name"),
            # "chineseName": category_name_map.get(category, ""),
            "items": items
        })


    # glob.glob("")
    # data  = [
    #     {
    #         "path":"reads-based-abundance-analysis2",
            
    #     }
    # ]
    return result


def get_pipeline_file(filename):
    nextflow_dict = get_all_pipeline()
    if filename not in nextflow_dict:
        raise HTTPException(status_code=500, detail=f"{filename}不存在!")  
    return nextflow_dict[filename]

def get_all_pipeline():
    pipeline_dir =  get_pipeline_dir()
    nextflow_list = glob.glob(f"{pipeline_dir}/*/nextflow/*.nf")
    nextflow_dict = {os.path.basename(item).replace(".nf",""):item for item in nextflow_list}
    return nextflow_dict



def get_downstream_analysis(item):
    with open(item,"r") as f:
        data = json.load(f)
    file_list = [
        item
        for d in data['items']
        if "downstreamAnalysis" in d
        for item in d['downstreamAnalysis']
    ]

    return file_list

@pipeline.get("/find_downstream_analysis/{analysis_method}",tags=['pipeline'])
def get_downstream_analysis_list(analysis_method):
    pipeline_files = get_pipeline_list()
    downstream_list = [get_downstream_analysis(item) for item in pipeline_files]
    downstream_list = [item for sublist in downstream_list for item in sublist]
    downstream_dict = {item['saveAnalysisMethod']: item for item in downstream_list  if 'saveAnalysisMethod' in item}
    return downstream_dict[analysis_method]
    



from fastapi import APIRouter,Depends,HTTPException
from sqlalchemy.orm import Session
# from brave.api.config.db import conn
from brave.api.schemas.sample import Sample
from typing import List
from starlette.status import HTTP_204_NO_CONTENT
from sqlalchemy import func, select
from brave.api.models.orm import SampleAnalysisResult
import glob
import importlib
import os
from brave.api.config.db import get_db_session
from sqlalchemy import and_,or_
import pandas as pd
from brave.api.models.core import samples
from brave.api.config.db import get_engine
from brave.api.schemas.analysis import AnalysisInput,Analysis
from typing import Dict, Any
from brave.api.models.core import analysis
import json
import importlib
import importlib.util
import uuid
import os
from brave.api.utils.get_db_utils import get_ids
from brave.api.config.config import get_settings
from brave.api.routes.pipeline import get_pipeline_file
import textwrap
from brave.api.routes.sample_result import find_analyais_result_by_ids
from brave.api.routes.sample_result import parse_result_one,parse_result_oneV2
from brave.api.service.pipeline  import get_all_module

analysis_api = APIRouter()


def update_or_save_result(db,project,sample_name,file_type,file_path,log_path,verison,analysis_name,software):
        sampleAnalysisResult = db.query(SampleAnalysisResult) \
        .filter(and_(SampleAnalysisResult.analysis_name == analysis_name,\
                SampleAnalysisResult.analysis_verison == verison, \
                SampleAnalysisResult.sample_name == sample_name, \
                SampleAnalysisResult.file_type == file_type, \
                SampleAnalysisResult.project == project \
            )).first()
        if sampleAnalysisResult:
            sampleAnalysisResult.contant_path = file_path
            sampleAnalysisResult.log_path = log_path
            sampleAnalysisResult.software = software
            db.commit()
            db.refresh(sampleAnalysisResult)
            print(">>>>更新: ",file_path,sample_name,file_type,log_path)
        else:
            sampleAnalysisResult = SampleAnalysisResult(analysis_name=analysis_name, \
                analysis_verison=verison, \
                sample_name=sample_name, \
                file_type=file_type, \
                log_path=log_path, \
                software=software, \
                project=project, \
                contant_path=file_path \
                    )
            db.add(sampleAnalysisResult)
            db.commit()
            print(">>>>新增: ",file_path,sample_name,file_type,log_path)


# def get_db_value(session, value):
#     ids = value
#     if not isinstance(value,list):
#         ids = [value]
#     analysis_result =  session.query(SampleAnalysisResult) \
#                 .filter(SampleAnalysisResult.id.in_(ids)) \
#                     .all()
                    
#     for item in analysis_result:
#         if item.content_type=="json" and not isinstance(item.content, dict):
#             item.content = json.loads(item.content)

#     if len(analysis_result)!=len(ids):
#         raise HTTPException(status_code=500, detail="数据存在问题!")
#     if not isinstance(value,list) and len(analysis_result)==1:
#         return analysis_result[0]
#     else:
#         return analysis_result
    



def parse_analysis(request_param,params_path,command_path,work_dir,module_name):
    all_module = get_all_module("py_parse_analysis")
    if module_name not in all_module:
        raise HTTPException(status_code=500, detail=f"py_parse_analysis: {module_name}没有找到!")
    py_module = all_module[module_name]
    # module_name = f'brave.api.parse_analysis.{module_name}'
    # if importlib.util.find_spec(module) is None:
    #     print(f"{module_name}不存在!")
    # else:
    module = importlib.import_module(py_module)
    get_db_field = getattr(module, "get_db_field")
    db_field = get_db_field()

    db_ids_dict = {key: get_ids(request_param[key]) for key in db_field if key in request_param}
    # with get_db_session() as session:
        # get_db_value
    db_dict = { key:find_analyais_result_by_ids(value) for key,value in  db_ids_dict.items()}
    parse_data = getattr(module, "parse_data")

    result = parse_data(request_param,db_dict)
    with open(params_path, "w") as f:
        json.dump(result,f)

        # get_script = getattr(module, "get_script")
        # script = get_script()
        # command = f"nextflow run -offline {script} -resume  -params-file {params_path} -w {work_dir} -with-trace trace.txt | tee .workflow.log"
        # with open(command_path, "w") as f:
        #     f.write(command)
        # get_output_format = getattr(module, "get_output_format")
        # output_format = get_output_format()
        # return json.dumps(output_format)

# ,response_model=List[Sample]
@analysis_api.post("/fast-api/save-analysis")
def save_analysis(request_param: Dict[str, Any]): # request_param: Dict[str, Any]
    # request_param = analysis_input.model_dump_json()
    

    with get_engine().begin() as conn:
        analysis_pipline = request_param['analysis_pipline']

        parse_analysis_module = request_param['parse_analysis_module']
        parse_analysis_result_module = json.dumps(request_param['parse_analysis_result_module'])
        new_analysis = {
            "project":request_param['project'],
            "analysis_name":request_param['analysis_name'],
            "analysis_method":analysis_pipline,
            "request_param":json.dumps(request_param),
            "parse_analysis_module":parse_analysis_module
        }
     
        output_dir=None
        work_dir=None
        result = None
        if "id" in request_param:
            stmt = select(analysis).where(analysis.c.id == request_param['id'])
            result = conn.execute(stmt).fetchone()
        if result:
            output_dir = result.output_dir
            work_dir = result.work_dir
            params_path = result.params_path
            command_path = result.command_path
            parse_analysis(request_param,params_path,command_path, work_dir,parse_analysis_module)
            new_analysis['output_format'] = parse_analysis_result_module
            stmt = analysis.update().values(new_analysis).where(analysis.c.id==request_param['id'])
        else:
            settings = get_settings()
            base_dir = settings.BASE_DIR
            work_dir = settings.WORK_DIR
            str_uuid = str(uuid.uuid4())
            # /ssd1/wy/workspace2/nextflow_workspace
            wrap_analysis_pipline = ""
            if 'wrap_analysis_pipeline' in request_param:
                wrap_analysis_pipline = request_param['wrap_analysis_pipeline']+"/"

            output_dir = f"{base_dir}/{request_param['project']}/{wrap_analysis_pipline}{analysis_pipline}"
            # /data/wangyang/nf_work/
            work_dir = f"{work_dir}/{request_param['project']}/{wrap_analysis_pipline}{analysis_pipline}"
            params_path = f"{output_dir}/params.json"
            command_path= f"{output_dir}/run.sh"
            if not os.path.exists(output_dir):
                os.makedirs(output_dir)
            if not os.path.exists(work_dir):
                os.makedirs(work_dir)
            # 写入脚本

            script =  f"{get_pipeline_file(analysis_pipline)}"
            new_analysis['pipeline_script'] = script

            command =  textwrap.dedent(f"""
            nextflow run -offline -resume  \\
                {script} \\
                -params-file {params_path} \\
                -w {work_dir} \\
                -with-trace trace.txt | tee .workflow.log
            """)
            with open(command_path, "w") as f:
                f.write(command)
            

            new_analysis['work_dir'] = work_dir
            new_analysis['output_dir'] = output_dir
            new_analysis['params_path'] = params_path
            new_analysis['command_path'] = command_path
            new_analysis['analysis_key'] = str_uuid

            parse_analysis(request_param,params_path, command_path,work_dir,parse_analysis_module)
            new_analysis['output_format'] = parse_analysis_result_module
      
            stmt = analysis.insert().values(new_analysis)
        conn.execute(stmt)
        
       
        

        # print()
        # return conn.execute(samples.select()).fetchall()
        # print()
    return {"msg":"success"}


@analysis_api.get(
    "/fast-api/analysis",
    response_model=List[Analysis],
)
def get_analysis(analysis_method,project):
    with get_engine().begin() as conn:
        return conn.execute(analysis.select().where(
            and_(analysis.c.analysis_method==analysis_method,
            analysis.c.project==project,
            )
        )).fetchall()


@analysis_api.delete("/fast-api/analysis/{id}",  status_code=HTTP_204_NO_CONTENT)
def delete_user(id: int):
    with get_engine().begin() as conn:
        conn.execute(analysis.delete().where(analysis.c.id == id))
    return {"message":"success"}


@analysis_api.post("/fast-api/parse-analysis-result/{id}")
def parse_analysis_result(id):
    with get_engine().begin() as conn:
        stmt = select(analysis).where(analysis.c.id == id)
        result = conn.execute(stmt).fetchone()
    output_format = json.loads(result.output_format)
    if len(output_format)==0:
        raise HTTPException(status_code=500, detail=f"分析{result.analysis_method}没有配置output_format!")
    for item in output_format:
        dir_path = f"{result.output_dir}/output/{item['dir']}"
        all_module = get_all_module("py_parse_analysis_result")
        if item['module'] not in all_module:
            raise HTTPException(status_code=500, detail=f"py_parse_analysis_result: {module_name}没有找到!")
        py_module = all_module[item['module']]
        module = importlib.import_module(py_module)
        # parse_result_one()
        moduleArgs = {}
        if "moduleArgs" in item:
            moduleArgs = item['moduleArgs']
        parse_result_oneV2(item['analysisMethod'],moduleArgs,result,module,dir_path,result.project,"V1.0",id)
    return {"message":"success"}


def get_ids(values):
    if isinstance(values, dict):
        if "sample" in values:
            return values['sample']
        elif "value" in values and  "label" in values:
            return values['value']
        else:
            return []
    return values


from typing import Optional
from pydantic import BaseModel

class ImportSample(BaseModel):
    content: Optional[str]


class Sample(BaseModel):
    id: Optional[int]
    sample_name: Optional[str]
    sequencing_target: Optional[str]
    sequencing_technique: Optional[str]
    sample_composition: Optional[str]
    fastq1: Optional[str]
    fastq2: Optional[str]

class UserCount(BaseModel):
    total: int

class SampleGroupQuery(BaseModel):
    project: Optional[str]
    sequencing_target: Optional[str]
    sequencing_technique: Optional[str]
    sample_composition: Optional[str]

class SampleGroup(BaseModel):
    id: Optional[int]
    sample_name: Optional[str]
    sample_key: Optional[str]
    sequencing_target: Optional[str]
    sequencing_technique: Optional[str]
    sample_composition: Optional[str]
    sample_group: Optional[str]
    sample_source: Optional[str]
    host_disease: Optional[str]
    project: Optional[str]

class Sample(BaseModel):
    id: Optional[int]
    sample_name: Optional[str]
    sample_key: Optional[str]
    sequencing_target: Optional[str]
    sequencing_technique: Optional[str]
    sample_composition: Optional[str]
    sample_group: Optional[str]
    sample_group_name: Optional[str]
    sample_source: Optional[str]
    host_disease: Optional[str]
    project: Optional[str]
    sample_individual: Optional[str]
    is_available: Optional[int]
    fastq1: Optional[str]
    fastq2: Optional[str]

class ProjectCount(BaseModel):
    project: Optional[str]
    count: Optional[int]


from setuptools import setup, find_packages

setup(
    name="pybrave",
    version="0.1.3",
    packages=find_packages(),
    install_requires=[
        "fastapi", 
        "sqlalchemy",
        "pandas",
        "uvicorn[standard]",
        "typer",
        "pymysql",
        "click==8.1.8"
        ],
    entry_points={
        "console_scripts": [
            "brave = brave.__main__:app", 
        ]
    },
    project_urls={                             
        "Source": "https://github.com/pybrave/brave",
        "Tracker": "https://github.com/pybrave/brave",
        "Documentation": "https://github.com/pybrave/brave",
    },
    author="WangYang",
    description="Bioinformatics Reactive Analysis and Visualization Engine",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    python_requires=">=3.7",
    package_data={
        "brave": [
            "frontend/**/*",  # 包含静态资源
            "pipeline/**/*"
        ]
    },
)



# brave/config.py

import os
from pathlib import Path
from functools import lru_cache
from importlib.resources import files
import sys
from pathlib import Path

# def get_pipeline_dir():
#     pipeline_config_path = str(files("brave.pipeline").joinpath("config.json"))
#     pipeline_dir = os.path.dirname(pipeline_config_path)
#     return  pipeline_dir
class Settings:
    def __init__(self):
        # 读取 base_dir
        home_dir = Path.home()
        
        default_base_dir= f"{home_dir}/.brave/base_dir"
        if not  os.path.exists(default_base_dir):    
            os.makedirs(default_base_dir) 
        base_dir = os.getenv("BASE_DIR",default_base_dir)
        self.BASE_DIR = Path(base_dir).resolve()# / "data"
        self.BASE_DIR.mkdir(parents=True, exist_ok=True)
        print(f"✅ Using BASE_DIR: {self.BASE_DIR}")

        default_databases_dir= f"{home_dir}/.brave/databases"
        if not  os.path.exists(default_databases_dir):    
            os.makedirs(default_databases_dir) 
        databases_dir = os.getenv("DATABASES_DIR",default_databases_dir)
        self.DATABASES_DIR = Path(databases_dir).resolve()# / "data"
        self.DATABASES_DIR.mkdir(parents=True, exist_ok=True)
        print(f"✅ Using DATABASES_DIR: {self.DATABASES_DIR}")
  

        work_dir = os.getenv("WORK_DIR",default_base_dir)
        self.WORK_DIR = Path(work_dir).resolve()# / "data"
        self.WORK_DIR.mkdir(parents=True, exist_ok=True)
        print(f"✅ Using WORK_DIR: {self.WORK_DIR}")

        # pipeline_dir_default = get_pipeline_dir()
       
        default_pipeline_dir = f"{home_dir}/.brave/pipeline"
        if not  os.path.exists(default_pipeline_dir):    
            os.makedirs(default_pipeline_dir)    
        pipeline_dir = os.getenv("PIPELINE_DIR",default_pipeline_dir)
        self.PIPELINE_DIR = Path(pipeline_dir).resolve()# / "data"
        # self.WORK_DIR.mkdir(parents=True, exist_ok=True)
        print(f"✅ Using PIPELINE_DIR: {self.PIPELINE_DIR}")
        if pipeline_dir not in sys.path:
            sys.path.insert(0, pipeline_dir)

        default_literature_dir = f"{home_dir}/.brave/literature"
        if not  os.path.exists(default_literature_dir):    
            os.makedirs(default_literature_dir)    
        literature_dir = os.getenv("LITERATURE_DIR",default_literature_dir)
        self.LITERATURE_DIR = Path(literature_dir).resolve()# / "data"
        # self.WORK_DIR.mkdir(parents=True, exist_ok=True)
        print(f"✅ Using LITERATURE_DIR: {self.LITERATURE_DIR}")
        # downstream_analysis_result = self.BASE_DIR / "downstream_analysis_result"
        # downstream_analysis_result.mkdir(parents=True, exist_ok=True)
        # self.DOWNSTREAM_ANALYSIS_RESULT_DIR = downstream_analysis_result
        # print(f"✅ Using DOWNSTREAM_ANALYSIS_RESULT_DIR: {self.DOWNSTREAM_ANALYSIS_RESULT_DIR}")


        # 读取数据库配置
        self.DB_TYPE = os.getenv("DB_TYPE", "sqlite").lower()
        if self.DB_TYPE == "mysql":
            MYSQL_URL = os.getenv("MYSQL_URL")
            self.DB_URL = f"mysql+pymysql://{MYSQL_URL}"
        else:
            self.DB_URL = f"sqlite:///{self.BASE_DIR / 'data.db'}"

        print(f"✅ Using DB_URL: {self.DB_URL}")

        


@lru_cache()
def get_settings() -> Settings:
    """全局共享 Settings 实例"""
    return Settings()



# https://github.com/FaztWeb/fastapi-mysql-restapi/blob/main/routes/user.py

from fastapi import FastAPI

from brave.api.routes.file_parse_plot import file_parse_plot
from brave.api.routes.sample_result import sample_result
from brave.api.routes.sample import sample
from brave.api.routes.analysis import analysis_api
from brave.api.routes.pipeline import pipeline
from brave.api.routes.literature import literature_api

from brave.api.routes.bio_database import bio_database
from fastapi.staticfiles import StaticFiles
from fastapi.responses import FileResponse
import os
from brave.api.config.config import get_settings

def create_app() -> FastAPI:
    app = FastAPI()
    frontend_path = os.path.join(os.path.dirname(__file__), "frontend")
    app.mount("/assets", StaticFiles(directory=os.path.join(frontend_path, "build","assets")), name="assets")
    # frontend_path = os.path.join(os.path.dirname(__file__), "frontend")

    app.mount("/brave-api/img", StaticFiles(directory=os.path.join(frontend_path, "img")), name="img")

    settings = get_settings()
    app.mount("/brave-api/dir", StaticFiles(directory=settings.BASE_DIR), name="base_dir")
    app.mount("/brave-api/literature/dir", StaticFiles(directory=os.path.join(settings.LITERATURE_DIR)), name="literature_dir")

    app.include_router(sample_result,prefix="/brave-api")
    app.include_router(file_parse_plot,prefix="/brave-api")
    app.include_router(sample,prefix="/brave-api")
    app.include_router(analysis_api,prefix="/brave-api")
    app.include_router(bio_database,prefix="/brave-api")
    app.include_router(pipeline,prefix="/brave-api")
    app.include_router(literature_api,prefix="/brave-api")

    @app.get("/favicon.ico")
    async def serve_frontend():
        favicon = os.path.join(frontend_path, "build/favicon.ico")
        return FileResponse(favicon)

    @app.get("/{full_path:path}")
    async def serve_frontend(full_path: str):
        index_path = os.path.join(frontend_path, "build/index.html")
        return FileResponse(index_path)

    return app
# @app.get("/")
# async def read_root():
#     time.sleep(10)
#     print("sleep")
#     print(threading.get_ident())
#     time.sleep(10)
#     print(threading.get_ident())
#     return {"Hello": "World"}
    
# @app.get("/abc")
# def read_root():
#     print("sleep")
#     print(threading.get_ident())
#     time.sleep(10)
#     return {"Hello": "World"}

# @app.get("/items/{item_id}")
# def read_item(item_id: int, q: Union[str, None] = None):
#     return {"item_id": item_id, "q": q}


from sqlalchemy import create_engine, MetaData
from sqlalchemy.orm import sessionmaker, declarative_base
from contextlib import contextmanager
import os
from brave.api.config.config import get_settings
import threading


# BASE_DIR = os.path.join(BASE_DIR, 'data')
# if not os.path.exists(BASE_DIR):
#     os.makedirs(BASE_DIR)
# print(f"Using BASE_DIR: {BASE_DIR}")
# DB_TYPE = os.getenv("DB_TYPE", "sqlite")  # "sqlite" or "mysql"
# if DB_TYPE == "mysql":
#     DB_URL = os.getenv("MYSQL_URL", "mysql+pymysql://root:123456@192.168.3.60:53306/pipeline")
# else:
#     # BASE_DIR = os.path.dirname(__file__)
#     DB_URL = f"sqlite:///{os.path.join(BASE_DIR, 'data.db')}"


# engine =  create_engine(DB_URL, echo=False)
# # engine = create_engine("mysql+pymysql://root:123456@192.168.3.60:53306/pipeline", echo=False)
# SessionLocal = sessionmaker(bind=engine, autocommit=False, autoflush=False)
meta = MetaData()
Base = declarative_base()
# class DbEngine:
#     def __init__(self):
# engine =  None
# SessionLocal = None
# engine_dict = {}

# 线程安全的 Engine 单例
class EngineSingleton:
    _engine = None
    _lock = threading.Lock()

    @classmethod
    def get_engine(cls):
        if cls._engine is None:
            with cls._lock:
                if cls._engine is None:  # 双重检查锁
                    settings = get_settings()

                    # cls._engine = create_engine(DATABASE_URL, connect_args={"check_same_thread": False})
                    cls._engine = create_engine(settings.DB_URL, echo=False, future=True)
        return cls._engine

# conn = engine.connect()
def init_engine():
    # global engine, SessionLocal
    # engine = create_engine(db_url, echo=False, future=True)
    engine = EngineSingleton.get_engine()
    
    # engine_dict.update({"aaa":"123"})
    return engine
    

def get_engine():
    engine = init_engine()
    return engine 



@contextmanager
def get_db_session():
    settings = get_settings()
    engine = init_engine()
    SessionLocal = sessionmaker(bind=engine, autoflush=False, autocommit=False)

    db = SessionLocal()
    
    try:
        yield db
        
    except:
        db.rollback()
        raise
    finally:
        db.close()
 


from fastapi import APIRouter
html_index = APIRouter()


@app.get("/{full_path:path}")
async def serve_frontend(full_path: str):
    index_path = os.path.join(frontend_path, "index.html")
    return FileResponse(index_path)


import json
import os
import glob
from brave.api.config.config import get_settings
from pathlib import Path


def get_pipeline_dir():
    settings = get_settings()
    return settings.PIPELINE_DIR

def get_pipeline_list():
    pipeline_dir =  get_pipeline_dir()
    pipeline_files = glob.glob(f"{pipeline_dir}/*/main.json")
    return pipeline_files
def get_module_name(item):
    pipeline_dir =  get_pipeline_dir()
    item = item.replace(f"{pipeline_dir}/","").replace("/",".")
    item = Path(item).stem 
    return item 
    # f'reads-alignment-based-abundance-analysis.py_plot.{module_name}'

def get_all_module(type):
    pipeline_dir =  get_pipeline_dir()
    nextflow_list = glob.glob(f"{pipeline_dir}/*/{type}/*.py")
    nextflow_dict = {os.path.basename(item).replace(".py",""):get_module_name(item) for item in nextflow_list}
    return nextflow_dict